package greenfoot.junitUtils;

import greenfoot.junitUtils.runner.TestRunnerState;

/**
 * It simulates string input for doing Test Driven Development (TDD) 
 * with JUnit. If we wants to verify the behaviour of any actor or 
 * world when it is stopped waiting Greenfoot.ask("..."), the test
 * can act as user and generates a possible out string. 
 * 
 * @author Francisco Guerra (francisco.guerra@ulpgc.es)
 */
public class Ask {

    /**
     * It queues a string in FIFO order for any next Greenfoot.ack("...") operation. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorAsk <b>extends</b> Actor {
	 *         <b>private</b> String ask;
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             ask = Greenfoot.ask("");
	 *         }
	 *
	 *         <b>public</b> String getAsk(){
	 *             <b>return</b> ask;
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the string that has been read by Greenfoot.ask("") operation:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.Ask;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorAskTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testAskAnyString() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorAsk actor = <b>new</b> ExampleActorAsk();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             Ask.set("AnyString");
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>("AnyString", actor.getAsk());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param nextAsk will be the string returned by the operation
     *                Greenfoot.ask("").
     *                
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void set(String nextAsk) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        TestRunnerState.setAsk(nextAsk, Object.class);
    }

	/**
	 * It queues a string in FIFO order for any next Greenfoot.ack("...") operation
	 * from objects of specific class.<br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorAsk <b>extends</b> Actor {
	 *         <b>private</b> String ask;
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             ask = Greenfoot.ask("");
	 *         }
	 *
	 *         <b>public</b> String getAsk(){
	 *             <b>return</b> ask;
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the string that has been read by Greenfoot.ask("") operation:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.Ask;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorAskTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testAskAnyString() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorAsk actor = <b>new</b> ExampleActorAsk();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             Ask.set("AnyString", ExampleActorAsk.<b>class</b>);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>("AnyString", actor.getAsk());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param nextAsk     will be the string returned by the operation
	 *                    Greenfoot.ask("").
	 *                    
	 * @param callerClass is the class of object that can receive the string queued.
	 *                    This class must be derived from Actor or World.
	 *                    
	 * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
	 *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
    public static void set(String nextAsk, Class<?> callerClass) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        TestRunnerState.setAsk(nextAsk, callerClass);
    }
}
