package greenfoot.junitUtils;

import greenfoot.core.WorldHandler;
import greenfoot.junitUtils.runner.TestRunnerState;
import javafx.scene.input.MouseButton;


/**
 * It simulates input events of keyboard or mouse for Test Driven Development (TDD) 
 * with JUnit. If we wants to verify the behaviour of any actor or world that receives
 * any keyboard key or mouse click, the test can generate this event and after verify 
 * the result. 
 * 
 * @author Francisco Guerra (francisco.guerra@ulpgc.es)
 */
public class EventDispatch {

    /**
     * It simulates a keyborad key that is typed into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorKeyboard <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.isKeyDown("g")){
     *                 setLocation(10, 20);
     *             }
     *             <b>if</b> (!Greenfoot.isKeyDown("g")){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.isKeyDown("g") 
	 * operation when the user has typed the key "g":
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorKeyboardTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testKeyTyped() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorKeyboard actor = <b>new</b> ExampleActorKeyboard();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.keyTyped("g");
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(100, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param key Any key of keyboard. The expected values are "up", "down",
     *              "left, "right" (cursor keys); "enter", "space", "tab", 
     *              "escape", "backspace", "F1" ... "F12" and "a" ... "z".
     *              
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void keyTyped(String key) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getKeyboardManager().setKey(key);
    }

    /**
     * It simulates a keyborad key that is pressed into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorKeyboard <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.isKeyDown("g")){
     *                 setLocation(10, 20);
     *             }
     *             <b>if</b> (!Greenfoot.isKeyDown("g")){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.isKeyDown("g") 
	 * operation when the user has pressed the key "g":
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorKeyboardTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testKeyPressed() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorKeyboard actor = <b>new</b> ExampleActorKeyboard();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.keyPressed("g");
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(10, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param key Any key of keyboard. The expected values are "up", "down",
     *              "left, "right" (cursor keys); "enter", "space", "tab", 
     *              "escape", "backspace", "F1" ... "F12" and "a" ... "z".
     *              
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void keyPressed(String key) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getKeyboardManager().pressKey(key);
    }

    /**
     * It simulates a keyborad key that is released into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorKeyboard <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.isKeyDown("g")){
     *                 setLocation(10, 20);
     *             }
     *             <b>if</b> (!Greenfoot.isKeyDown("g")){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.isKeyDown("g") operation 
	 * when the user has pressed the key "g" and then the user has released this key:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorKeyboardTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testKeyReleased() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorKeyboard actor = <b>new</b> ExampleActorKeyboard();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.keyPressed("g");
	 *             EventDispatch.keyReleased("g");
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(100, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param key Any key of keyboard. The expected values are "up", "down",
     *              "left, "right" (cursor keys); "enter", "space", "tab", 
     *              "escape", "backspace", "F1" ... "F12" and "a" ... "z".
     *              
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void keyReleased(String key) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getKeyboardManager().releaseKey(key);
    }

    /**
     * It removes all events of the keyboard buffer. It not need be called when runOnce operation is used. 
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
	public static void keyboardExited() throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
		WorldHandler.getInstance().getKeyboardManager().clearLatchedKeys();				
	}
	
	/**
	 * It simulates that a mouse botton has been clicked into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseClicked(this)){
     *                 move(6);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseClicked(this) 
	 * operation when the user has clicked with a mouse botton over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseClicked() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseClicked(1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(7, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mouseClicked(int x, int y) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
	    WorldHandler.getInstance().getMouseManager().mouseClicked(x(x), y(y), MouseButton.NONE, 1);	    
        WorldHandler.getInstance().getMouseManager().newActStarted();
	}

	/**
	 * It simulates that a mouse botton has been clicked into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseClicked(this)){
     *                 move(6);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseClicked(this) 
	 * operation when the user has clicked with a mouse botton over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseClicked() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseClicked(1,1,1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(7, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work
	 * 
     * @param button clicked (0: NONE; 1: left button; 2: middle button; 3: right button)
     * 
     * @param nClickes is the click count recorded.
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mouseClicked(int x, int y, int button, int nClickes) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseClicked(x(x),
                                                                  y(y), 
                                                                  MouseButton.values()[button], 
                                                                  nClickes);
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}

	/**
	 * It simulates that a mouse botton has been pressed into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mousePressed(this)){
     *                 setLocation(10, 20);
     *             }
	 *             <b>if</b> (!Greenfoot.mousePressed(this)){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mousePressed(this) 
	 * operation when the user has pressed with a mouse botton over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMousePressed() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mousePressed(1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(10, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mousePressed(int x, int y) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mousePressed(x(x), y(y), MouseButton.NONE);
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}

	/**
	 * It simulates that a mouse botton has been pressed into the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mousePressed(this)){
     *                 setLocation(10, 20);
     *             }
	 *             <b>if</b> (!Greenfoot.mousePressed(this)){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mousePressed(this) 
	 * operation when the user has pressed with a mouse botton over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMousePressed() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mousePressed(1,1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(10, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work
	 * 
     * @param button pressed (0: NONE; 1: left button; 2: middle button; 3: right button)
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mousePressed(int x, int y, int button) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mousePressed(x(x), y(y), MouseButton.values()[button]);
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}

	/**
     * It simulates that a mouse button has been dragged over the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseDragged(this){
     *                 move(6);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseDragged(this) 
	 * operation when the user has dragged with the mouse over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseDragged() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseDragged(1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(7, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param x Location index on the x-axis where the even has triggered into the current work
     * 
     * @param y Location index on the y-axis where the even has triggered into the current work
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void mouseDragged(int x, int y) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseDragged(x(x), y(y), MouseButton.NONE);
        WorldHandler.getInstance().getMouseManager().newActStarted();       
    }

    /**
     * It simulates that a mouse button has been dragged over the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseDragged(this){
     *                 move(6);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseDragged(this) 
	 * operation when the user has dragged with the mouse over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseDragged() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseDragged(1,1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(7, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
     * 
     * @param x Location index on the x-axis where the even has triggered into the current work
     * 
     * @param y Location index on the y-axis where the even has triggered into the current work
     * 
     * @param button dragged (0: NONE; 1: left button; 2: middle button; 3: right button)
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
     */
    public static void mouseDragged(int x, int y, int button) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseDragged(x(x), y(y), MouseButton.values()[button]);
        WorldHandler.getInstance().getMouseManager().newActStarted();       
    }

	/**
	 * It simulates that a mouse botton is released into the current world.  <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseDragged(this)){
     *                 setLocation(10, 20);
     *             }
	 *             <b>if</b> (Greenfoot.mouseDragEnded(this)){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseDragEnded(this) 
	 * operation when the user dragges and after releases with the mouse over 
	 * this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseDragEnded() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseDragged(1,1);
	 *             EventDispatch.mouseReleased(1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(100, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mouseReleased(int x, int y) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseReleased(x(x), y(y), MouseButton.NONE);
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}
	
	/**
	 * It simulates that a mouse botton is released into the current world.  <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseDragged(this)){
     *                 setLocation(10, 20);
     *             }
	 *             <b>if</b> (Greenfoot.mouseDragEnded(this)){
     *                 setLocation(100, 20);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseDragEnded(this) 
	 * operation when the user dragges and after releases with the mouse over 
	 * this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseDragEnded() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseDragged(1,1,1);
	 *             EventDispatch.mouseReleased(1,1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(100, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work.
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work.
	 * 
     * @param button released (0: NONE; 1: left button; 2: middle button; 3: right button)
     * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mouseReleased(int x, int y, int button) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseReleased(x(x), y(y), MouseButton.values()[button]);
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}

	/**
	 * It simulates that the mouse is moved around the current world. <br>
	 * <br>
	 * For the next actor behaviour:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>public class</b> ExampleActorMouse <b>extends</b> Actor {
	 *	
	 *         <i>@Override</i>
	 *         <b>public void</b> act(){
	 *             <b>if</b> (Greenfoot.mouseMoved(this){
     *                 move(6);
     *             }
	 *         }
	 *     } 
	 * </code>
	 * </pre>
	 * 
	 * The next test verifies the behaviour for Greenfoot.mouseMoved(this) 
	 * operation when the user moves the mouse over this actor:
	 * 
	 * <pre>
	 *  <code>
	 *     <b>import</b> ...
	 *     <b>import</b> greenfoot.TestRunnerState.EventDispatch;
	 *     <b>import</b> greenfoot.TestRunnerState.WorldCreator;
	 *     
	 *     <i>@RunWith</i>(GreenfootRunner.class)
	 *     <b>public class</b> ExampleActorMouseTest {
	 *         <i>@Test</i>
	 *         <b>public void</b> testMouseMoved() <b>throws</b> Exception {
	 *             <b>// GIVEN </b>
	 *             World world = WorldCreator.getWorld(400, 300, 1);
	 *             ExampleActorMouse actor = <b>new</b> ExampleActorMouse();
	 *             world.addObject(actor, 1, 1);
	 *             
	 *             <b>// WHEN </b>
	 *             EventDispatch.mouseMoved(1,1);
	 *             WorldCreator.runOnce(world);
	 *             
	 *             <b>// THEN </b>
	 *             <i>assertEquals</i>(7, actor.getX());
	 *         }
	 *     }
	 * </code>
	 * </pre>
	 * 
	 * @param x Location index on the x-axis where the even has triggered into the current work
	 * 
	 * @param y Location index on the y-axis where the even has triggered into the current work
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mouseMoved(int x, int y) throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseMoved(x(x), y(y));
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}	

	/**
	 * It removes all events of the mouse buffer. It not need be called when runOnce operation is used.
	 * 
     * @throws GreenfootRunnerExpected is raised when <i>@RunWith</i>(GreenfootRunner.class) 
     *         or <i>@ExtendWith</i>(GreenfootRunner.class) is not found.
	 */
	public static void mouseExited() throws GreenfootRunnerExpected{
    	TestRunnerState.assertRunningUnitTest();
        WorldHandler.getInstance().getMouseManager().mouseExited();
        WorldHandler.getInstance().getMouseManager().newActStarted();		
	}
	
	private static int x(int x) {
        int cellSize = WorldHandler.getInstance().getWorld().getCellSize();
        return x*cellSize + cellSize/2;
	}

    private static int y(int y) {
        int cellSize = WorldHandler.getInstance().getWorld().getCellSize();
        return y*cellSize + cellSize/2;
    }

}
